/*
 * Skeleton-code behorende bij het college Netwerken, opleiding Informatica,
 * Universiteit Leiden.
 */

#ifndef __PACKET_BUFFER_H__
#define __PACKET_BUFFER_H__

#include "variables.h"

#include <pthread.h>
#include <stdbool.h>
#include <stdint.h>

/* Packet format definition. Feel free to make changes as necessary. */
#define MYSTRING_SIZE 32

enum packet_type_t
{
  PACKET_TYPE_DATA,
  PACKET_TYPE_RR
};

struct packet_t
{
  enum packet_type_t type;
  uint8_t seq_number;

  char mystring[MYSTRING_SIZE];
  short length;
};

/* Packet buffer. We use this to decouple buffer management from the
 * operating system.
 */
struct packet_buffer_t
{
  struct packet_t buffer[PACKET_BUFFER_SIZE];

  pthread_mutex_t mutex;
  pthread_cond_t cond_taken;
  pthread_cond_t cond_put;

  int n_packets;
  int max_utilization;
  int in_pos;
  int out_pos;
};

void packet_buffer_init(struct packet_buffer_t *packet_buffer);
bool packet_buffer_empty(struct packet_buffer_t *packet_buffer);
void packet_buffer_put(struct packet_buffer_t *packet_buffer,
                       struct packet_t *packet,
                       const float drop_perc);
bool packet_buffer_get(struct packet_buffer_t *packet_buffer,
                       struct packet_t *packet,
                       const long timeout_msecs);

#endif /* __PACKET_BUFFER_H__ */
